@@ -51,27 +51,25 @@ package com.openexchange.apps.manifests.json;
 
 import java.util.Arrays;
 import java.util.Collection;
-import org.json.JSONArray;
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXActionServiceFactory;
-import com.openexchange.apps.manifests.json.osgi.ServerConfigServicesLookup;
-import com.openexchange.exception.OXException;
 import com.openexchange.server.ServiceLookup;
 
 /**
  * {@link ManifestActionFactory}
  *
  * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
+ * @author <a href="mailto:marc.arens@open-xchange.com">Marc Arens</a>
  */
 public class ManifestActionFactory implements AJAXActionServiceFactory {
 
     private final AJAXActionService all;
     private final ConfigAction config;
 
-    public ManifestActionFactory(ServiceLookup services, JSONArray manifests, ServerConfigServicesLookup registry) {
+    public ManifestActionFactory(ServiceLookup services, ManifestBuilder manifestBuilder) {
         super();
-        all = new AllAction(services, manifests, registry);
-        config = new ConfigAction(services, manifests, registry);
+        all = new AllAction(manifestBuilder);
+        config = new ConfigAction(services, manifestBuilder);
     }
 
     @Override
@@ -80,7 +78,7 @@ public class ManifestActionFactory implements AJAXActionServiceFactory {
     }
 
     @Override
-    public AJAXActionService createActionService(String action) throws OXException {
+    public AJAXActionService createActionService(String action) {
         if (action.equals("config")) {
             return config;
         }
