@@ -54,6 +54,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.servlet.ServletException;
@@ -75,69 +76,43 @@ import com.openexchange.tools.session.ServerSessionAdapter;
  */
 public class AppsLoadServlet extends SessionServlet {
 
-    public static FileContributor contributors = null;
-
     private static final long serialVersionUID = -8909104490806162791L;
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(AppsLoadServlet.class);
 
     private static String ZONEINFO = "io.ox/core/date/tz/zoneinfo/";
 
-    private final FileCache appCache, tzCache;
+    // ----------------------------------------------------------------------------------------------------------------------
+
+    private final AtomicReference<FileCache> appCacheReference;
+    private final AtomicReference<FileCache> tzCacheReference;
+    private final AtomicReference<FileContributor> fileContributorReference;
 
     /**
      * Initializes a new {@link AppsLoadServlet}.
      *
-     * @throws IOException
+     * @param roots The app roots
+     * @param zoneinfo The zone information
+     * @param contributor The (composite) file contributor.
+     * @throws IOException If canonical path names of given files cannot be determined
      */
-    public AppsLoadServlet(final File[] roots, final File zoneinfo) throws IOException {
+    public AppsLoadServlet(File[] roots, File zoneinfo, FileContributor contributor) throws IOException {
         super();
-        appCache = new FileCache(roots);
-        tzCache = new FileCache(zoneinfo);
+        fileContributorReference = new AtomicReference<FileContributor>(contributor);
+        appCacheReference = new AtomicReference<FileCache>(new FileCache(roots));
+        tzCacheReference = new AtomicReference<FileCache>(new FileCache(zoneinfo));
     }
 
-    private static Pattern moduleRE = Pattern.compile("(?:/(text|raw);)?([\\w/+-]+(?:\\.[\\w/+-]+)*)");
-
-    private static Pattern escapeRE = Pattern.compile("[\\x00-\\x1f'\\\\\\u2028\\u2029]");
-
-    private static String[] escapes = {
-        "\\\\x00", "\\\\x01", "\\\\x02", "\\\\x03", "\\\\x04", "\\\\x05", "\\\\x06", "\\\\x07", "\\\\b", "\\\\t", "\\\\n", "\\\\v",
-        "\\\\f", "\\\\r", "\\\\x0e", "\\\\x0f", "\\\\x10", "\\\\x11", "\\\\x12", "\\\\x13", "\\\\x14", "\\\\x15", "\\\\x16", "\\\\x17",
-        "\\\\x18", "\\\\x19", "\\\\x1a", "\\\\x1b", "\\\\x1c", "\\\\x1d", "\\\\x1e", "\\\\x1f" };
-
-    private static byte[] SUFFIX;
-    static {
-        try {
-            SUFFIX = "\n/*:oxsep:*/\n".getBytes(Charsets.UTF_8_NAME);
-        } catch (UnsupportedEncodingException e) {
-            SUFFIX = "\n/*:oxsep:*/\n".getBytes();
-        }
-    }
-
-    private static void escape(final CharSequence s, final StringBuffer sb) {
-        final Matcher e = escapeRE.matcher(s);
-        while (e.find()) {
-            final int chr = e.group().codePointAt(0);
-            String replacement;
-            switch (chr) {
-            case 0x27:
-                replacement = "\\\\'";
-                break;
-            case 0x5c:
-                replacement = "\\\\\\\\";
-                break;
-            case 0x2028:
-                replacement = "\\\\u2028";
-                break;
-            case 0x2029:
-                replacement = "\\\\u2029";
-                break;
-            default:
-                replacement = escapes[chr];
-            }
-            e.appendReplacement(sb, replacement);
-        }
-        e.appendTail(sb);
+    /**
+     * Reinitializes this Servlet using given arguments
+     *
+     * @param roots The app roots
+     * @param zoneinfo The zone information
+     * @throws IOException If canonical path names of given files cannot be determined
+     */
+    public void reinitialize(File[] roots, File zoneinfo) throws IOException {
+        appCacheReference.set(new FileCache(roots));
+        tzCacheReference.set(new FileCache(zoneinfo));
     }
 
     private String escapeName(String name) {
@@ -163,16 +138,13 @@ public class AppsLoadServlet extends SessionServlet {
     private class ErrorWriter {
 
         private boolean buffering = true;
-
         private final HttpServletResponse resp;
-
         private OutputStream out;
-
         private byte[][] buffer;
-
         private int count = 0;
 
-        public ErrorWriter(HttpServletResponse resp, int length) {
+        ErrorWriter(HttpServletResponse resp, int length) {
+            super();
             this.resp = resp;
             this.buffer = new byte[length][];
         }
@@ -254,7 +226,7 @@ public class AppsLoadServlet extends SessionServlet {
             boolean isTZ = name.startsWith(ZONEINFO);
             final String resolved = isTZ ? name.substring(ZONEINFO.length()) : name;
 
-            FileCache cache = isTZ ? tzCache : appCache;
+            FileCache cache = isTZ ? tzCacheReference.get() : appCacheReference.get();
             byte[] data = cache.get(module, new FileCache.Filter() {
 
                 @Override
@@ -289,6 +261,7 @@ public class AppsLoadServlet extends SessionServlet {
             } else {
                 // Try external sources
                 String options = "{}";
+                FileContributor contributors = fileContributorReference.get();
                 if (contributors != null) {
                     try {
                         FileContribution contribution = contributors.getData(ServerSessionAdapter.valueOf(session), module);
@@ -343,4 +316,50 @@ public class AppsLoadServlet extends SessionServlet {
 
     }
 
+    // ------------------------------------------------------------------------------------------------------------------------
+
+    private static Pattern moduleRE = Pattern.compile("(?:/(text|raw);)?([\\w/+-]+(?:\\.[\\w/+-]+)*)");
+
+    private static Pattern escapeRE = Pattern.compile("[\\x00-\\x1f'\\\\\\u2028\\u2029]");
+
+    private static String[] escapes = {
+        "\\\\x00", "\\\\x01", "\\\\x02", "\\\\x03", "\\\\x04", "\\\\x05", "\\\\x06", "\\\\x07", "\\\\b", "\\\\t", "\\\\n", "\\\\v",
+        "\\\\f", "\\\\r", "\\\\x0e", "\\\\x0f", "\\\\x10", "\\\\x11", "\\\\x12", "\\\\x13", "\\\\x14", "\\\\x15", "\\\\x16", "\\\\x17",
+        "\\\\x18", "\\\\x19", "\\\\x1a", "\\\\x1b", "\\\\x1c", "\\\\x1d", "\\\\x1e", "\\\\x1f" };
+
+    private static byte[] SUFFIX;
+    static {
+        try {
+            SUFFIX = "\n/*:oxsep:*/\n".getBytes(Charsets.UTF_8_NAME);
+        } catch (UnsupportedEncodingException e) {
+            SUFFIX = "\n/*:oxsep:*/\n".getBytes();
+        }
+    }
+
+    private static void escape(final CharSequence s, final StringBuffer sb) {
+        final Matcher e = escapeRE.matcher(s);
+        while (e.find()) {
+            final int chr = e.group().codePointAt(0);
+            String replacement;
+            switch (chr) {
+            case 0x27:
+                replacement = "\\\\'";
+                break;
+            case 0x5c:
+                replacement = "\\\\\\\\";
+                break;
+            case 0x2028:
+                replacement = "\\\\u2028";
+                break;
+            case 0x2029:
+                replacement = "\\\\u2029";
+                break;
+            default:
+                replacement = escapes[chr];
+            }
+            e.appendReplacement(sb, replacement);
+        }
+        e.appendTail(sb);
+    }
+
 }
